"""                                                                                 Provisioning Agent.                                                                 Scan tree at                                                                          /machines                                                                         """
from twisted.internet.defer import inlineCallbacks
from twisted.internet import service

from ensemble.environment import Environment
from txzookeeper.client import ZookeeperClient
from logging import getLogger
import yaml
import zookeeper

log = getLogger("ensemble.provisioning")


class ProvisioningAgent(service.Service):

    machine_path = "/machine/"

    def __init__(self):
        self.client = None
        self.machines = []

    def startService(self):
        log.info("Starting Machine Provisioning Agent")
        self.client = ZookeeperClient("127.0.0.1:2181")
        connect_deferred = self.client.connect()
        connect_deferred.addCallback(self.on_startup_get_provider)
        connect_deferred.addCallback(self.on_startup_scan_tree)

    def stopService(self):
        self.client.close()

    @inlineCallbacks
    def on_startup_get_provider(self, *ignored):
        data = yield self.client.get('/environment')
        environment = yaml.load(data)
        self.environment = Environment('default', environment)
        self.provider = environment.get_machine_provider()

    @inlineCallbacks
    def on_startup_scan_tree(self, *ignored):
        """ When the agent first launches it scans the machine tree
        looking for machines that need to be launched.
        """
        machines = yield self.client.get_children(
            self.machine_path, self.on_machine_modified)
        for machine in machines:
            try:
                info_path = '%s/%s/info' % (self.machine_path,
                                            machine)
                print "looking for",info_path
                yield self.client.get(info_path)
            except zookeeper.NoNodeException:
                # XXX: locking is good
                #with self.client.get_lock('...'):
                pass
            else:
                continue

            print "starting new machine", machine
            machine = yield self.provider.start_machine()
            md = machine.serialize()
            yield self.client.create(info_path, md)

    def on_machine_modified(self, event):
        """ When a machine is added/removed.
        """
        # Not handling removal for now.
        if event.event_type == 'removed':
            return

        if event.event_type == 'added':
            pass


def main():
    pass


application = service.Application('provisioning_agent')
agent = ProvisioningAgent()
agent.setServiceParent(application)
