from twisted.internet.defer import DeferredList, inlineCallbacks
from twisted.internet import reactor
from ensemble.formula.repository import LocalFormulaRepository
from ensemble.environment import config
import logging
import os
import yaml
import zookeeper

log = logging.getLogger('hackenv')


@inlineCallbacks
def command(options):
    """
    Deploy a formula to Ensemble!
    """
    environment = options.environments.get_default()
    provider = environment.get_machine_provider()
    env_data = dict(provider.config)

    env_data.update(dict(
        secret_key=os.environ['AWS_SECRET_ACCESS_KEY'],
        access_key=os.environ['AWS_ACCESS_KEY_ID']
        ))

    zk_client = yield provider.connect()
    try:
        yield zk_client.delete('/environment')
    except zookeeper.NoNodeException:
        pass

    yield zk_client.create('/environment', yaml.dump(env_data))

    reactor.stop()

