import time
import logging

from ensemble.environment.errors import EnvironmentsConfigError
from ensemble.control import setup_logging, main

from .common import ControlToolTest


class ControlInitiailizationTest(ControlToolTest):

    # The EnvironmentTestBase will replace our $HOME, so that tests will
    # write properly to a temporary directory.
    def test_write_sample_config(self):
        """
        When ensemble-control is called without a valid environment
        configuration, it should write one down and raise an error to
        let the user know it should be edited.
        """
        try:
            main([])
        except EnvironmentsConfigError, error:
            self.assertTrue(error.sample_written)
        else:
            self.fail("EnvironmentsConfigError not raised")


class ControlLoggingTest(ControlToolTest):

    def test_logging_format(self):
        setup_logging(False)

        # ensure that we use gtm regardless of system settings
        logging.getLogger().handlers[0].formatter.converter = time.gmtime

        record = logging.makeLogRecord(
            {'created': 0, 'msecs': 0, 'levelno': logging.INFO})
        logging.getLogger().handle(record)
        self.assertEqual(
            self.log.getvalue(),
            '1970-01-01 00:00:00,000 Level None \n')

    def test_default_logging(self):
        """
        Default log-level is informational.
        """
        setup_logging(False)
        root = logging.getLogger()
        name = logging.getLevelName(root.getEffectiveLevel())
        self.assertEqual(name, 'INFO')

    def test_verbose_logging(self):
        """
        When verbose logging is enabled, the log level is set to debugging.
        """
        setup_logging(True)
        root = logging.getLogger()
        self.assertEqual(logging.getLevelName(root.level), 'DEBUG')
        custom = logging.getLogger('custom')
        self.assertEqual(custom.getEffectiveLevel(), root.getEffectiveLevel())
