import hashlib
from zipfile import ZipFile
from zipfile import BadZipfile

from .info import FormulaInfo
from ..errors import FormulaError


class FormulaBundle(object):
    """ZIP-archive that contains formula directory content."""

    def __init__(self, path):
        try:
            zf = ZipFile(path, 'r')
        except BadZipfile, exc:
            raise FormulaError(path, "Must be a ZIP-file (%s)." % str(exc))

        info = FormulaInfo()
        try:
            metadata = zf.read('metadata.yaml')
        except KeyError:
            raise FormulaError(path,
                               "Archive does not contain required file: "
                               "\"metadata.yaml\".")

        info.parse(metadata)
        self.info = info
        self.path = path

    def as_bundle(self):
        return self

    def get_digest(self):
        hasher = hashlib.new('sha256')
        with open(self.path, 'r') as fp:
            hasher.update(fp.read())
        return hasher.hexdigest()

