"""
Validate AWS Assumptions.
"""

"""
Validate assumptions regarding s3 api usage via txaws.

Requirements for functional test
 - valid amazon credentials present in the environment as
   AWS_SECRET_ACCESS_KEY, AWS_ACCESS_KEY_ID

"""

import random
from twisted.internet.defer import inlineCallbacks
from txaws.service import AWSServiceRegion
from ensemble.lib.testing import TestCase


class AWSSecurityGroupTest(TestCase):

    def setUp(self):
        region = AWSServiceRegion()
        self.ec2 = region.get_ec2_client()
        self.security_group = "ensemble-test-%s" % (random.random())

    @inlineCallbacks
    def tearDown(self):
        yield self.ec2.delete_security_group(self.security_group)

    @inlineCallbacks
    def test_create_and_get_group(self):
        """Verify input/outputs of creating a security group.

        Specifically we want to see if we can pick up the owner id off the
        group, which we need for group 2 group ec2 authorizations."""
        data = yield self.ec2.create_security_group(
            self.security_group, "test")
        self.assertEqual(data, True)

        info = yield self.ec2.describe_security_groups(self.security_group)
        self.assertEqual(len(info), 1)
        group = info.pop()
        self.assertTrue(group.owner_id)

    @inlineCallbacks
    def test_create_and_authorize_group(self):

        yield self.ec2.create_security_group(self.security_group, "test")
        info = yield self.ec2.describe_security_groups(self.security_group)
        group = info.pop()
        yield self.ec2.authorize_security_group(
            self.security_group,
            source_group_name = self.security_group,
            source_group_owner_id = group.owner_id)

        info = yield self.ec2.describe_security_groups(self.security_group)
        group = info.pop()
        self.assertEqual(group.name, self.security_group)
