"""
Validate assumptions regarding s3 api usage via txaws.

Requirements for functional test
 - valid amazon credentials present in the environment as
   AWS_SECRET_ACCESS_KEY, AWS_ACCESS_KEY_ID

"""

import random
from twisted.internet.defer import inlineCallbacks
from twisted.python.failure import Failure
from txaws.service import AWSServiceRegion
from txaws.s3.exception import S3Error

from ensemble.lib.testing import TestCase




class S3FilesTest(TestCase):

    def setUp(self):
        region = AWSServiceRegion()
        self.s3 = region.get_s3_client()
        self.control_bucket = "ensemble-test-%s" % (random.random())
        return self.s3.create_bucket(self.control_bucket)

    @inlineCallbacks
    def tearDown(self):
        listing = yield self.s3.get_bucket(self.control_bucket)
        for ob in listing.contents:
            yield self.s3.delete_object(self.control_bucket, ob.key)
        yield self.s3.delete_bucket(self.control_bucket)

    def test_put_object(self):
        """Verify input/outputs of putting an object in the bucket.

        The output is just an empty string on success."""
        d = self.s3.put_object(
            self.control_bucket, "pirates/gold.txt", "blah blah")

        def verify_result(result):
            self.assertEqual(result, "")

        d.addCallback(verify_result)
        return d

    @inlineCallbacks
    def test_get_object(self):
        """Verify input/outputs of getting an object from the bucket."""

        yield self.s3.put_object(
            self.control_bucket, "pirates/ship.txt", "argh argh")
        ob = yield self.s3.get_object(
            self.control_bucket, "pirates/ship.txt")
        self.assertEqual(ob, "argh argh")

    def test_get_object_nonexistant(self):
        """Verify output when an object does not exist."""

        d = self.s3.get_object(self.control_bucket, "pirates/treasure.txt")

        def verify_result(result):
            self.assertTrue(isinstance(result, Failure))
            result.trap(S3Error)

        d.addBoth(verify_result)
        return d
